package com.idiot.operationbackend.controller.group;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.idiot.operationbackend.entity.AccountCustomerMsg;
import com.idiot.operationbackend.entity.CustomerMsg;
import com.idiot.operationbackend.service.facade.AccountCustomerMsgService;
import com.idiot.operationbackend.service.facade.AuthUserService;
import com.idiot.operationbackend.service.facade.CustomerMsgService;
import com.idiot.operationbackend.support.Constants;
import com.idiot.operationbackend.support.JsonResult;
import com.idiot.operationbackend.util.JwtTokenUtil;
import com.idiot.operationbackend.vo.BaseType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 客服消息
 * @author wang xiao
 * @date Created in 14:48 2020/9/22
 */
@RestController
@RequestMapping("/customer")
@Api(value = "CustomerMsgController", tags ="客服消息")
public class CustomerMsgController {

    private final Logger logger = LoggerFactory.getLogger(CustomerMsgController.class);

    @Autowired
    private AuthUserService userService;

    @Autowired
    private CustomerMsgService customerMsgService;

    @Autowired
    private AccountCustomerMsgService accountCustomerMsgService;


    @GetMapping
    @ApiOperation(value = "查询客服消息")
    public ResponseEntity<JsonResult<Page<CustomerMsg>>> pageCustomerMsg (@RequestHeader String token,
                                                                          @RequestParam int page,
                                                                          @RequestParam(required = false) String status,
                                                                          @RequestParam(required = false) String startDate,
                                                                          @RequestParam(required = false) String endDate) {
        String userId = JwtTokenUtil.getUserId(token);
        String authId = userService.queryAuthId(userId);
        logger.info("用户：{}查询客服消息列表 page:{},authId:{}",userId,page,authId);
        Page<CustomerMsg> customerMsgPage = customerMsgService.queryPageCustomerMsg(status,page,startDate,endDate,authId);
        return ResponseEntity.ok(JsonResult.success(customerMsgPage));

    }

    @GetMapping("/{msgId}")
    @ApiOperation(value = "该客服消息下公众号发送详情")
    public ResponseEntity<JsonResult<List<AccountCustomerMsg>>> accountMsg (@PathVariable String msgId) {

        logger.info("查询客服消息公众号详情列表，nsgId:{}",msgId);
        List<AccountCustomerMsg> accountCustomerMsgs = accountCustomerMsgService.queryByMsgId(msgId);
        return ResponseEntity.ok(JsonResult.success(accountCustomerMsgs));
    }

    @GetMapping("/type")
    @ApiOperation(value = "客服消息状态码")
    public ResponseEntity<JsonResult<List<BaseType>>> customerStatus () {

        List<BaseType> baseTypes = new ArrayList<>(4);
        BaseType success  = new BaseType(Constants.SUCCESSED,"发送成功");
        BaseType waiting  = new BaseType(Constants.WAITING,"未发送");
        BaseType run  = new BaseType(Constants.RUNING,"发送中");
        BaseType end  = new BaseType(Constants.END,"发送中止");
        BaseType fail  = new BaseType(Constants.FAILD,"发送失败");
        baseTypes.add(success);
        baseTypes.add(waiting);
        baseTypes.add(run);
        baseTypes.add(end);
        baseTypes.add(fail);
        return ResponseEntity.ok(JsonResult.success(baseTypes));
    }



}
